==> ruby-date2-2.1.tgz: lib/ruby/site_ruby/1.6/date3.rb <==
# date3.rb: Written by Tadayoshi Funaba 1998-2000
# $Id: date3.rb,v 1.10 2000-07-16 10:23:45+09 tadf Exp $

require 'rational'

class Date

  include Comparable

  IDENT = 3

  MONTHNAMES = [nil] + %w(January February March April May June July
			  August September October November December)

  DAYNAMES = %w(Sunday Monday Tuesday Wednesday Thursday Friday Saturday)

  ITALY     = 2299161 # 1582-10-15
  ENGLAND   = 2361222 # 1752-09-14
  JULIAN    = false
  GREGORIAN = true

  class << self

    def os? (jd, sg)
      case sg
      when Numeric; jd < sg
      else;         not sg
      end
    end

    def ns? (jd, sg) not os?(jd, sg) end

    def civil_to_jd(y, m, d, sg=GREGORIAN)
      if m <= 2
	y -= 1
	m += 12
      end
      a = (y / 100.0).floor
      b = 2 - a + (a / 4.0).floor
      jd = (365.25 * (y + 4716)).floor +
	(30.6001 * (m + 1)).floor +
	d + b - 1524
      if os?(jd, sg)
	jd -= b
      end
      jd
    end

    def jd_to_civil(jd, sg=GREGORIAN)
      if os?(jd, sg)
	a = jd
      else
	x = ((jd - 1867216.25) / 36524.25).floor
	a = jd + 1 + x - (x / 4.0).floor
      end
      b = a + 1524
      c = ((b - 122.1) / 365.25).floor
      d = (365.25 * c).floor
      e = ((b - d) / 30.6001).floor
      dom = b - d - (30.6001 * e).floor
      if e <= 13
	m = e - 1
	y = c - 4716
      else
	m = e - 13
	y = c - 4715
      end
      return y, m, dom
    end

    def ordinal_to_jd(y, d, sg=GREGORIAN)
      civil_to_jd(y, 1, d, sg)
    end

    def jd_to_ordinal(jd, sg=GREGORIAN)
      y = jd_to_civil(jd, sg)[0]
      doy = jd - civil_to_jd(y - 1, 12, 31, ns?(jd, sg))
      return y, doy
    end

    def jd_to_commercial(jd, sg=GREGORIAN)
      ns = ns?(jd, sg)
      a = jd_to_civil(jd - 3, ns)[0]
      y = if jd >= commercial_to_jd(a + 1, 1, 1, ns) then a + 1 else a end
      w = 1 + (jd - commercial_to_jd(y, 1, 1, ns)) / 7
      d = (jd + 1) % 7
      if d.zero? then d = 7 end
      return y, w, d
    end

    def commercial_to_jd(y, w, d, ns=GREGORIAN)
      jd = civil_to_jd(y, 1, 4, ns)
      (jd - (((jd - 1) + 1) % 7)) +
	7 * (w - 1) +
	(d - 1)
    end

    def clfloor(x, y=1)
      q, r = x.divmod(y)
      q = q.to_i
      return q, r
    end

    def rjd_to_jd(rjd, of=0) clfloor(rjd + of + 1.to_r/2) end
    def jd_to_rjd(jd, fr, of=0) jd + fr - of - 1.to_r/2 end

    def fr1_to_time(fr)
      h,   fr = clfloor(fr, 1.to_r/24)
      min, fr = clfloor(fr, 1.to_r/1440)
      s,   fr = clfloor(fr, 1.to_r/86400)
      return h, min, s, fr
    end

    def time_to_fr1(h, min, s)
      h.to_r/24 + min.to_r/1440 + s.to_r/86400
    end

    def mjd_to_jd(mjd) mjd + 4800001.to_r/2 end
    def jd_to_mjd(jd) jd - 4800001.to_r/2 end
    def tjd_to_jd(tjd) tjd + 4880001.to_r/2 end
    def jd_to_tjd(jd) jd - 4880001.to_r/2 end
    def tjd2_to_jd(cycle, tjd) tjd_to_jd(cycle * 10000 + tjd) end
    def jd_to_tjd2(jd) clfloor(jd_to_tjd(jd), 10000) end
    def ld_to_jd(ld) ld + 2299160 end
    def jd_to_ld(jd) jd - 2299160 end

    def jd_to_wday(jd) (jd + 1) % 7 end

    def julian_leap? (y) y % 4 == 0 end
    def gregorian_leap? (y) y % 4 == 0 and y % 100 != 0 or y % 400 == 0 end

    alias_method :leap?, :gregorian_leap?

    alias_method :new0, :new

    def new1(jd=0, sg=ITALY) new0(jd, 0, sg) end

    def exist3? (y, m, d, sg=ITALY)
      if m < 0
	m += 13
      end
      if d < 0
	ny, nm = clfloor(y * 12 + m, 12)
	nm,    = clfloor(m + 1, 1)
	la = nil
	31.downto 1 do |z|
	  break if la = exist3?(y, m, z, sg)
	end
	ns = ns?(la, sg)
	d = jd_to_civil(civil_to_jd(ny, nm, 1, ns) + d, ns)[-1]
      end
      jd = civil_to_jd(y, m, d, sg)
      return unless [y, m, d] == jd_to_civil(jd, sg)
      jd
    end

    alias_method :exist?, :exist3?

    def new3(y=-4712, m=1, d=1, sg=ITALY)
      unless jd = exist3?(y, m, d, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(jd, 0, sg)
    end

    alias_method :new, :new3

    def exist2? (y, d, sg=ITALY)
      if d < 0
	ny = y + 1
	la = nil
	366.downto 1 do |z|
	  break if la = exist2?(y, z, sg)
	end
	ns = ns?(la, sg)
	d = jd_to_ordinal(ordinal_to_jd(ny, 1, ns) + d, ns)[-1]
      end
      jd = ordinal_to_jd(y, d, sg)
      return unless [y, d] == jd_to_ordinal(jd, sg)
      jd
    end

    def new2(y=-4712, d=1, sg=ITALY)
      unless jd = exist2?(y, d, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(jd, 0, sg)
    end

    def existw? (y, w, d, sg=ITALY)
      if d < 0
	d += 8
      end
      if w < 0
	w = jd_to_commercial(commercial_to_jd(y + 1, 1, 1) + w * 7)[1]
      end
      jd = commercial_to_jd(y, w, d)
      return unless ns?(jd, sg)
      return unless [y, w, d] == jd_to_commercial(jd)
      jd
    end

    def neww(y=1582, w=41, d=5, sg=ITALY)
      unless jd = existw?(y, w, d, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(jd, 0, sg)
    end

    def today(sg=ITALY)
      new0(civil_to_jd(*(Time.now.to_a[3..5].reverse << sg)), 0, sg)
    end

    def new1t(jd=0, fr=0, of=0, sg=ITALY)
      new0(jd_to_rjd(jd, fr, of), of, sg)
    end

    def existt? (h, min, s)
      h   += 24 if h   < 0
      min += 60 if min < 0
      s   += 60 if s   < 0
      return unless (0..29) === h and
		    (0..59) === min and
		    (0..59) === s
      time_to_fr1(h, min, s)
    end

    def exist3t? (y, m, d, h, min, s, of=0, sg=ITALY)
      return unless jd = exist3?(y, m, d, sg)
      return unless fr = existt?(h, min, s)
      jd_to_rjd(jd, fr, of)
    end

    def new3t(y=-4712, m=1, d=1, h=0, min=0, s=0, of=0, sg=ITALY)
      unless rjd = exist3t?(y, m, d, h, min, s, of, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(rjd, of, sg)
    end

    def exist2t? (y, d, h, min, s, of=0, sg=ITALY)
      return unless jd = exist2?(y, d, sg)
      return unless fr = existt?(h, min, s)
      jd_to_rjd(jd, fr, of)
    end

    def new2t(y=-4712, d=1, h=0, min=0, s=0, of=0, sg=ITALY)
      unless rjd = exist2t?(y, d, h, min, s, of, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(rjd, of, sg)
    end

    def existwt? (y, w, d, h, min, s, of=0, sg=ITALY)
      return unless jd = existw?(y, w, d)
      return unless fr = existt?(h, min, s)
      jd_to_rjd(jd, fr, of)
    end

    def newwt(y=-1582, w=41, d=5, h=0, min=0, s=0, of=0, sg=ITALY)
      unless rjd = existwt?(y, w, d, h, min, s, of, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(rjd, of, sg)
    end

    def now(sg=ITALY)
      i = Time.now
      a = i.to_a[0..5].reverse
      if a[-1] == 60 then a[-1] -= 1 end
      d = Time.gm(*a).to_i - Time.local(*a).to_i
      d += d / d.abs
      of = (d / 60).to_r/1440
      jd = civil_to_jd(*(a[0,3] << sg))
      fr = time_to_fr1(*(a[3,3]))
      rjd = jd_to_rjd(jd, fr, of)
      us = i.usec.to_r/86400000000
      new0(rjd + us, of, sg)
    end

    def once(*ids)
      for id in ids
	module_eval <<-"end;"
	  alias_method :__#{id.to_i}__, :#{id.id2name}
	  def #{id.id2name}(*args, &block)
	    (@__#{id.to_i}__ ||= [__#{id.to_i}__(*args, &block)])[0]
	  end
	end;
      end
    end

    private :once

  end

  def initialize(rjd=0, of=0, sg=ITALY) @rjd, @of, @sg = rjd, of, sg end

  def rjd() @rjd end
  def rmjd() type.jd_to_mjd(@rjd) end
  def rtjd() type.jd_to_tjd(@rjd) end
  def rtjd2() type.jd_to_tjd2(@rjd) end

  once :rmjd, :rtjd, :rtjd2

  def jd() type.rjd_to_jd(@rjd, @of)[0] end
  def fr1() type.rjd_to_jd(@rjd, @of)[1] end
  def mjd() type.jd_to_mjd(jd) end
  def tjd() type.jd_to_tjd(jd) end
  def tjd2() type.jd_to_tjd2(jd) end
  def ld() type.jd_to_ld(jd) end

  once :jd, :fr1, :mjd, :tjd, :tjd2, :ld

  def civil() type.jd_to_civil(jd, @sg) end
  def ordinal() type.jd_to_ordinal(jd, @sg) end
  def commercial() type.jd_to_commercial(jd, @sg) end

  once :civil, :ordinal, :commercial
  private :civil, :ordinal, :commercial

  def year() civil[0] end
  def yday() ordinal[1] end
  def mon() civil[1] end

  alias_method :month, :mon

  def mday() civil[2] end

  alias_method :day, :mday

  def time() type.fr1_to_time(fr1) end

  once :time
  private :time

  def hour() time[0] end
  def min() time[1] end
  def sec() time[2] end
  def fr2() time[3] end

  def cwyear() commercial[0] end
  def cweek() commercial[1] end
  def cwday() commercial[2] end

  def wday() type.jd_to_wday(jd) end

  once :wday

  def os? () type.os?(jd, @sg) end
  def ns? () type.ns?(jd, @sg) end

  once :os?, :ns?

  def leap?
    type.jd_to_civil(type.civil_to_jd(year, 3, 1, ns?) - 1,
		     ns?)[-1] == 29
  end

  once :leap?

  def sg() @sg end
  def newsg(sg=type::ITALY) type.new0(@rjd, @of, sg) end

  def italy() newsg(type::ITALY) end
  def england() newsg(type::ENGLAND) end
  def julian() newsg(type::JULIAN) end
  def gregorian() newsg(type::GREGORIAN) end

  def of() @of end
  def newof(of=0) type.new0(@rjd, of, @sg) end

  def utc() newof(0) end

  alias_method :gmt, :utc

  def zone
    ['Z',
      format('%+.2d%02d',
	     (of     / (1.to_r/24)).to_i,
	     (of.abs % (1.to_r/24) / (1.to_r/1440)).to_i)
    ][of<=>0]
  end

  def + (n)
    case n
    when Numeric; return type.new0(@rjd + n, @of, @sg)
    end
    raise TypeError, 'expected numeric'
  end

  def - (x)
    case x
    when Numeric; return type.new0(@rjd - x, @of, @sg)
    when Date;    return @rjd - x.rjd
    end
    raise TypeError, 'expected numeric or date'
  end

  def <=> (other)
    case other
    when Numeric; return @rjd <=> other
    when Date;    return @rjd <=> other.rjd
    end
    raise TypeError, 'expected numeric or date'
  end

  def === (other)
    case other
    when Numeric; return jd == other
    when Date;    return jd == other.jd
    end
    raise TypeError, 'expected numeric or date'
  end

  def >> (n)
    y, m = type.clfloor(year * 12 + (mon - 1) + n, 12)
    m,   = type.clfloor(m + 1, 1)
    d = mday
    d -= 1 until jd2 = type.exist3?(y, m, d, ns?)
    self + (jd2 - jd)
  end

  def << (n) self >> -n end

  def step(limit, step)
    da = self
    op = [:-,:<=,:>=][step<=>0]
    while da.__send__(op, limit)
      yield da
      da += step
    end
    self
  end

  def upto(max, &block) step(max, +1, &block) end
  def downto(min, &block) step(min, -1, &block) end

  def succ() self + 1 end

  alias_method :next, :succ

  def eql? (other) Date === other and self == other end
  def hash() type.clfloor(@rjd)[0] end

  def inspect() format('#<%s: %s,%s,%s>', type, @rjd, @of, @sg) end

  def to_s
    format('%.4d-%02d-%02dT%02d:%02d:%02d,%d%s',
	   year, mon, mday, hour, min, sec, (fr2 * 864000).to_i, zone)
  end

  def _dump(limit) Marshal.dump([@rjd, @of, @sg], -1) end
  def self._load(str) new0(*Marshal.load(str)) end

end

class Integer

  def fortnight() (self * 14).to_r/1 end

  alias_method :fortnights, :fortnight

  def week() (self * 7).to_r/1 end

  alias_method :weeks, :week
  alias_method :sennight, :week
  alias_method :sennights, :week

  def day() self.to_r/1 end

  alias_method :days, :day

  def hour() self.to_r/24 end

  alias_method :hours, :hour

  def minute() self.to_r/1440 end

  alias_method :minutes, :minute

  def second() self.to_r/86400 end

  alias_method :seconds, :second

  def millisecond() self.to_r/86400000 end

  alias_method :milliseconds, :millisecond

  def microsecond() self.to_r/86400000000 end

  alias_method :microseconds, :microsecond

end

==> ruby-date2-2.4.3.tgz: lib/ruby/site_ruby/1.6/date3.rb <==
# date3.rb: Written by Tadayoshi Funaba 1998-2001
# $Id: date3.rb,v 1.13 2001-02-13 01:13:07+09 tadf Exp $

require 'rational'

class Date

  include Comparable

  IDENT = 3

  MONTHNAMES = [nil] + %w(January February March April May June July
			  August September October November December)

  DAYNAMES = %w(Sunday Monday Tuesday Wednesday Thursday Friday Saturday)

  ITALY     = 2299161 # 1582-10-15
  ENGLAND   = 2361222 # 1752-09-14
  JULIAN    = false
  GREGORIAN = true

  class << self

    def os? (jd, sg)
      case sg
      when Numeric; jd < sg
      else;         not sg
      end
    end

    def ns? (jd, sg) not os?(jd, sg) end

    def civil_to_jd(y, m, d, sg=GREGORIAN)
      if m <= 2
	y -= 1
	m += 12
      end
      a = (y / 100.0).floor
      b = 2 - a + (a / 4.0).floor
      jd = (365.25 * (y + 4716)).floor +
	(30.6001 * (m + 1)).floor +
	d + b - 1524
      if os?(jd, sg)
	jd -= b
      end
      jd
    end

    def jd_to_civil(jd, sg=GREGORIAN)
      if os?(jd, sg)
	a = jd
      else
	x = ((jd - 1867216.25) / 36524.25).floor
	a = jd + 1 + x - (x / 4.0).floor
      end
      b = a + 1524
      c = ((b - 122.1) / 365.25).floor
      d = (365.25 * c).floor
      e = ((b - d) / 30.6001).floor
      dom = b - d - (30.6001 * e).floor
      if e <= 13
	m = e - 1
	y = c - 4716
      else
	m = e - 13
	y = c - 4715
      end
      return y, m, dom
    end

    def ordinal_to_jd(y, d, sg=GREGORIAN)
      civil_to_jd(y, 1, d, sg)
    end

    def jd_to_ordinal(jd, sg=GREGORIAN)
      y = jd_to_civil(jd, sg)[0]
      doy = jd - civil_to_jd(y - 1, 12, 31, ns?(jd, sg))
      return y, doy
    end

    def jd_to_commercial(jd, sg=GREGORIAN)
      ns = ns?(jd, sg)
      a = jd_to_civil(jd - 3, ns)[0]
      y = if jd >= commercial_to_jd(a + 1, 1, 1, ns) then a + 1 else a end
      w = 1 + (jd - commercial_to_jd(y, 1, 1, ns)) / 7
      d = (jd + 1) % 7
      if d.zero? then d = 7 end
      return y, w, d
    end

    def commercial_to_jd(y, w, d, ns=GREGORIAN)
      jd = civil_to_jd(y, 1, 4, ns)
      (jd - (((jd - 1) + 1) % 7)) +
	7 * (w - 1) +
	(d - 1)
    end

    def clfloor(x, y=1)
      q, r = x.divmod(y)
      q = q.to_i
      return q, r
    end

    def rjd_to_jd(rjd, of=0) clfloor(rjd + of + 1.to_r/2) end
    def jd_to_rjd(jd, fr, of=0) jd + fr - of - 1.to_r/2 end

    def fr1_to_time(fr)
      h,   fr = clfloor(fr, 1.to_r/24)
      min, fr = clfloor(fr, 1.to_r/1440)
      s,   fr = clfloor(fr, 1.to_r/86400)
      return h, min, s, fr
    end

    def time_to_fr1(h, min, s)
      h.to_r/24 + min.to_r/1440 + s.to_r/86400
    end

    def mjd_to_jd(mjd) mjd + 4800001.to_r/2 end
    def jd_to_mjd(jd) jd - 4800001.to_r/2 end
    def tjd_to_jd(tjd) tjd + 4880001.to_r/2 end
    def jd_to_tjd(jd) jd - 4880001.to_r/2 end
    def tjd2_to_jd(cycle, tjd) tjd_to_jd(cycle * 10000 + tjd) end
    def jd_to_tjd2(jd) clfloor(jd_to_tjd(jd), 10000) end
    def ld_to_jd(ld) ld + 2299160 end
    def jd_to_ld(jd) jd - 2299160 end

    def jd_to_wday(jd) (jd + 1) % 7 end

    def julian_leap? (y) y % 4 == 0 end
    def gregorian_leap? (y) y % 4 == 0 and y % 100 != 0 or y % 400 == 0 end

    alias_method :leap?, :gregorian_leap?

    alias_method :new0, :new

    def existt? (h, min, s)
      h   += 24 if h   < 0
      min += 60 if min < 0
      s   += 60 if s   < 0
      return unless (0..29) === h and
		    (0..59) === min and
		    (0..59) === s
      time_to_fr1(h, min, s)
    end

    private :existt?

    def exist1? (jd, h, min, s, of, sg=ITALY)
      return unless fr = existt?(h, min, s)
      jd_to_rjd(jd, fr, of)
    end

    def new1(jd=0, h=0, min=0, s=0, of=0, sg=ITALY)
      unless rjd = exist1?(jd, h, min, s, of, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(rjd, of, sg)
    end

    def exist3? (y, m, d, h, min, s, of, sg=ITALY)
      if m < 0
	m += 13
      end
      if d < 0
	ny, nm = clfloor(y * 12 + m, 12)
	nm,    = clfloor(nm + 1, 1)
	jd = civil_to_jd(ny, nm, d + 1, sg)
	ns = ns?(jd, sg)
	return unless [y, m] == jd_to_civil(jd, sg)[0..1]
	return unless [ny, nm, 1] == jd_to_civil(jd - d, ns)
      else
	jd = civil_to_jd(y, m, d, sg)
	return unless [y, m, d] == jd_to_civil(jd, sg)
      end
      return unless fr = existt?(h, min, s)
      jd_to_rjd(jd, fr, of)
    end

    alias_method :exist?, :exist3?

    def new3(y=-4712, m=1, d=1, h=0, min=0, s=0, of=0, sg=ITALY)
      unless rjd = exist3?(y, m, d, h, min, s, of, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(rjd, of, sg)
    end

    alias_method :new, :new3

    def exist2? (y, d, h, min, s, of, sg=ITALY)
      if d < 0
	ny, = clfloor(y + 1, 1)
	jd = ordinal_to_jd(ny, d + 1, sg)
	ns = ns?(jd, sg)
	return unless [y] == jd_to_ordinal(jd, sg)[0..0]
	return unless [ny, 1] == jd_to_ordinal(jd - d, ns)
      else
	jd = ordinal_to_jd(y, d, sg)
	return unless [y, d] == jd_to_ordinal(jd, sg)
      end
      return unless fr = existt?(h, min, s)
      jd_to_rjd(jd, fr, of)
    end

    def new2(y=-4712, d=1, h=0, min=0, s=0, of=0, sg=ITALY)
      unless rjd = exist2?(y, d, h, min, s, of, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(rjd, of, sg)
    end

    def existw? (y, w, d, h, min, s, of, sg=ITALY)
      if d < 0
	d += 8
      end
      if w < 0
	w = jd_to_commercial(commercial_to_jd(y + 1, 1, 1) + w * 7)[1]
      end
      jd = commercial_to_jd(y, w, d)
      return unless ns?(jd, sg)
      return unless [y, w, d] == jd_to_commercial(jd)
      return unless fr = existt?(h, min, s)
      jd_to_rjd(jd, fr, of)
    end

    def neww(y=-1582, w=41, d=5, h=0, min=0, s=0, of=0, sg=ITALY)
      unless rjd = existw?(y, w, d, h, min, s, of, sg)
	raise ArgumentError, 'invalid date'
      end
      new0(rjd, of, sg)
    end

    def now(sg=ITALY)
      i = Time.now
      a = i.to_a[0..5].reverse
      if a[-1] == 60 then a[-1] -= 1 end
      d = Time.gm(*a).to_i - Time.local(*a).to_i
      d += d / d.abs
      of = (d / 60).to_r/1440
      jd = civil_to_jd(*(a[0,3] << sg))
      fr = time_to_fr1(*(a[3,3]))
      rjd = jd_to_rjd(jd, fr, of)
      us = i.usec.to_r/86400000000
      new0(rjd + us, of, sg)
    end

    def once(*ids)
      for id in ids
	module_eval <<-"end;"
	  alias_method :__#{id.to_i}__, :#{id.id2name}
	  private :__#{id.to_i}__
	  def #{id.id2name}(*args, &block)
	    (@__#{id.to_i}__ ||= [__#{id.to_i}__(*args, &block)])[0]
	  end
	end;
      end
    end

    private :once

  end

  def initialize(rjd=0, of=0, sg=ITALY) @rjd, @of, @sg = rjd, of, sg end

  def rjd() @rjd end
  def rmjd() type.jd_to_mjd(@rjd) end
  def rtjd() type.jd_to_tjd(@rjd) end
  def rtjd2() type.jd_to_tjd2(@rjd) end

  once :rmjd, :rtjd, :rtjd2

  def jd() type.rjd_to_jd(@rjd, @of)[0] end
  def fr1() type.rjd_to_jd(@rjd, @of)[1] end
  def mjd() type.jd_to_mjd(jd) end
  def tjd() type.jd_to_tjd(jd) end
  def tjd2() type.jd_to_tjd2(jd) end
  def ld() type.jd_to_ld(jd) end

  once :jd, :fr1, :mjd, :tjd, :tjd2, :ld

  def civil() type.jd_to_civil(jd, @sg) end
  def ordinal() type.jd_to_ordinal(jd, @sg) end
  def commercial() type.jd_to_commercial(jd, @sg) end

  once :civil, :ordinal, :commercial
  private :civil, :ordinal, :commercial

  def year() civil[0] end
  def yday() ordinal[1] end
  def mon() civil[1] end

  alias_method :month, :mon

  def mday() civil[2] end

  alias_method :day, :mday

  def time() type.fr1_to_time(fr1) end

  once :time
  private :time

  def hour() time[0] end
  def min() time[1] end
  def sec() time[2] end
  def fr2() time[3] end

  def cwyear() commercial[0] end
  def cweek() commercial[1] end
  def cwday() commercial[2] end

  def wday() type.jd_to_wday(jd) end

  once :wday

  def os? () type.os?(jd, @sg) end
  def ns? () type.ns?(jd, @sg) end

  once :os?, :ns?

  def leap?
    type.jd_to_civil(type.civil_to_jd(year, 3, 1, ns?) - 1,
		     ns?)[-1] == 29
  end

  once :leap?

  def sg() @sg end
  def newsg(sg=type::ITALY) type.new0(@rjd, @of, sg) end

  def italy() newsg(type::ITALY) end
  def england() newsg(type::ENGLAND) end
  def julian() newsg(type::JULIAN) end
  def gregorian() newsg(type::GREGORIAN) end

  def of() @of end
  def newof(of=0) type.new0(@rjd, of, @sg) end

  def utc() newof(0) end

  alias_method :gmt, :utc

  def zone
    ['Z',
      format('%+.2d%02d',
	     (of     / (1.to_r/24)).to_i,
	     (of.abs % (1.to_r/24) / (1.to_r/1440)).to_i)
    ][of<=>0]
  end

  def + (n)
    case n
    when Numeric; return type.new0(@rjd + n, @of, @sg)
    end
    raise TypeError, 'expected numeric'
  end

  def - (x)
    case x
    when Numeric; return type.new0(@rjd - x, @of, @sg)
    when Date;    return @rjd - x.rjd
    end
    raise TypeError, 'expected numeric or date'
  end

  def <=> (other)
    case other
    when Numeric; return @rjd <=> other
    when Date;    return @rjd <=> other.rjd
    end
    raise TypeError, 'expected numeric or date'
  end

  def === (other)
    case other
    when Numeric; return jd == other
    when Date;    return jd == other.jd
    end
    raise TypeError, 'expected numeric or date'
  end

  def >> (n)
    y, m = type.clfloor(year * 12 + (mon - 1) + n, 12)
    m,   = type.clfloor(m + 1, 1)
    d = mday
    d -= 1 until jd2 = type.exist3?(y, m, d, ns?)
    self + (jd2 - jd)
  end

  def << (n) self >> -n end

  def step(limit, step)
    da = self
    op = [:-,:<=,:>=][step<=>0]
    while da.__send__(op, limit)
      yield da
      da += step
    end
    self
  end

  def upto(max, &block) step(max, +1, &block) end
  def downto(min, &block) step(min, -1, &block) end

  def succ() self + 1 end

  alias_method :next, :succ

  def eql? (other) Date === other and self == other end
  def hash() type.clfloor(@rjd)[0] end

  def inspect() format('#<%s: %s,%s,%s>', type, @rjd, @of, @sg) end

  def to_s
    format('%.4d-%02d-%02dT%02d:%02d:%02d,%d%s',
	   year, mon, mday, hour, min, sec, (fr2 * 864000).to_i, zone)
  end

  def _dump(limit) Marshal.dump([@rjd, @of, @sg], -1) end
  def self._load(str) new0(*Marshal.load(str)) end

end

if Object.const_defined? :IRB

  class Integer

    def fortnight() (self * 14).to_r/1 end

    alias_method :fortnights, :fortnight

    def week() (self * 7).to_r/1 end

    alias_method :weeks, :week
    alias_method :sennight, :week
    alias_method :sennights, :week

    def day() self.to_r/1 end

    alias_method :days, :day

    def hour() self.to_r/24 end

    alias_method :hours, :hour

    def minute() self.to_r/1440 end

    alias_method :minutes, :minute

    def second() self.to_r/86400 end

    alias_method :seconds, :second

    def millisecond() self.to_r/86400000 end

    alias_method :milliseconds, :millisecond

    def microsecond() self.to_r/86400000000 end

    alias_method :microseconds, :microsecond

  end

end
